import React, { useRef, useEffect, useState, useLayoutEffect } from "react";
import { useGoogleMapsLibraries } from "google-maps-api-react-provider";
import MapProvider from "./provider/map-provider";
import useMap from "./hooks/useMap";

import "./map.css";

interface MapProps extends google.maps.MapOptions {
  className?: string;
  children?: React.ReactElement | React.ReactNode | React.JSX.Element;
  onClick?: (e: google.maps.MapMouseEvent) => void;
}

const Map = (props: MapProps) => {
  const {
    className: classNameProp = "",
    children,
    onClick,
    zoom = 14,
    ...options
  } = props;

  const { googleLibraries, addLibrary } = useGoogleMapsLibraries();

  const { mapsLibrary } = googleLibraries;

  const mapElementRef = useRef<HTMLDivElement>(null);
  const [map, setMap] = useState<google.maps.Map | null>(null);

  const className = `map ${classNameProp}`;

  useLayoutEffect(() => {
    addLibrary("maps");
  }, []);

  useEffect(() => {
    if (mapsLibrary && mapElementRef.current) {
      mapsLibrary.then(({ Map }) => {
        //@ts-ignore
        setMap(new Map(mapElementRef.current, { zoom, ...options }));
      });
    }
  }, [mapsLibrary, mapElementRef]);

  useEffect(() => {
    if (map) {
      map.addListener("click", (e: google.maps.MapMouseEvent) => {
        if (onClick) onClick(e);
      });
    }
  }, [map]);

  return (
    <MapProvider value={map}>
      <div ref={mapElementRef} className={className}>
        {children}
      </div>
    </MapProvider>
  );
};

export default Map;
export { useMap };
